# -*- coding: utf8 -*-
from __future__ import unicode_literals

import base64
from driverng import Worklist
from driverng.message import BinGraphContainer
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class KaluzaGraphContainer(BinGraphContainer):

    def __init__(self, data=None):
        super(KaluzaGraphContainer, self).__init__()
        self["data"] = data
        self["format"] = "jpeg"

    def get_bytes(self):
        return bytes(self.data)


class KaluzaSegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    sender_id = HprimString((5, 2))
    software_revision = HprimString((5, 3))
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class KaluzaSegmentP(Segment):
    laboratory_patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician_id = HprimString((14, 1))
    attending_physician_lastname = HprimString((14, 2))
    attending_physician_firstname = HprimString((14, 3))
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class KaluzaSegmentO(Segment):
    specimen_id = HprimString(3)
    priority = HprimString(6)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    specimen_type = HprimString(16)
    ordering_physician_id = HprimString((17, 1))
    ordering_physician_lastname = HprimString((17, 2))
    ordering_physician_firstname = HprimString((17, 3))
    analyzer_name = HprimString((21, 1))
    sub_module_name = HprimString((21, 2))
    report_type = HprimString(26)

    results = HprimForeign("R", "order")
    infos = HprimForeign("M", "infos")
    comments = HprimForeign("C", "order")


class KaluzaSegmentR(Segment):
    test_id = HprimString((3, 1))
    test_name = HprimString((3, 2))
    test_id_type = HprimString((3, 3))
    local_code = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)
    reference_range = HprimString(6)
    result_status = HprimString(9)
    date_test_completed = HprimDate(13)
    serial_number = HprimString((14, 1))
    instrument_model = HprimString((14, 2))
    manufacturer = HprimString((14, 3))

    infos = HprimForeign("M", "infos")


class KaluzaSegmentM(Segment):
    image_name = HprimString((3, 4))
    data = HprimString(4)
    status = HprimString(9)
    date = HprimDate(13)


class KaluzaSegmentC(Segment):
    comment_text = HprimString(4)


class KaluzaSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "kaluza",
        "label": "KALUZA ASTM Monodirectionnel BECKMAN-COULTER",
    }
    ENCODING = "utf8"
    SEGMENT_CLASS_MAP = {
        "H": KaluzaSegmentH,
        "P": KaluzaSegmentP,
        "O": KaluzaSegmentO,
        "R": KaluzaSegmentR,
        "M": KaluzaSegmentM,
        "C": KaluzaSegmentC,
        "L": KaluzaSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = KaluzaSegmentH(type_segment="H", **kwargs)
        message.end = Segment(type_segment="L")
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        control_lot=seg_o.specimen_id,
                        sample_id=seg_o.specimen_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        type="patient",
                    )
                for seg_m in seg_o.infos:
                    sample.results.add(
                        code=seg_m.image_name,
                        value="GRAPHE",
                        graph=KaluzaGraphContainer(
                            data=base64.b64decode(seg_m.data),
                        )
                    )
                for seg_r in seg_o.results:
                    comments = []
                    for seg_c in seg_r.commentaires:
                        comments.append(seg_c.comment_text)
                    sample.results.add(
                        code=seg_r.local_code,
                        value=seg_r.value,
                        comments=[],
                        date=seg_r.date_test_completed,
                        instrument=seg_r.serial_number,
                    )
        return worklist
