# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
from six import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String, Multiple


class ShimadzuBase(Splitted):
    SEPARATOR = "\t"


class ShimadzuOrder(ShimadzuBase):
    FIELDS = (
        String("analysis"),
        String("tray_name"),
        String("vial"),
        String("sample_name"),
        String("sample_id"),
        String("sample_type"),
        String("analysis_type"),
        String("method"),
        String("data_file"),
        String("background"),
        String("baseline_data_file"),
        String("level"),
        String("volume"),
    )


class ShimadzuResultExport(object):
    FIELDS = [
        String("filename"),
        String("sample_name"),
        String("sample_id"),
        Multiple("results", (
            String("resu"),
        ), separator=";"),
    ]

    def parse(self, line):
        len_line = len(line)
        fields = self.FIELDS[:len_line]
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class ShimadzuMultiple(list):

    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: ShimadzuOrder,
        MessageV2.RESULT: ShimadzuResultExport,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        result = data.replace(u'\xa0', '')
        if len(result) == 10 and "0:Unknown" in data:
            obj = self.add(type_message=MessageV2.ORDER)
        elif "Flag ID	Data Filename" in data:
            buf = StringIO(result)
            reader = csv.reader(buf, delimiter=str('\t'))
            for line in reader:
                if not len(line):
                    continue
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)
        else:
            buf = StringIO(result)
            reader = csv.reader(buf, delimiter=str(';'))
            for line in reader:
                if not len(line):
                    continue
                # Traitement du fichier Export Clam de type csv
                if len(line) > 4:
                    obj = self.add(type_message=MessageV2.RESULT)
                    obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "shimadzu",
        "label": "Lab Solution Shimadzu"
    }
    OPTIONS = [
        MessageV2.Option(
            "shimadzu_qc_prefix",
            label="Préfixe QC",
            default="",
        ),
        MessageV2.Option(
            "shimadzu_analyseur",
            label="Indiquer les noms des instruments présents dans le premier champ des résultats séparés par des \";\"",
            default="UPLC1;UPLC2",
        )
    ]

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: ShimadzuOrder,
        MessageV2.RESULT: ShimadzuResultExport,
    }

    def __init__(self, **kwargs):
        self.qc_prefix = kwargs.pop("shimadzu_qc_prefix", "")
        self.code_analyseur = kwargs.pop("shimadzu_analyseur", "UPLC1;UPLC2")
        super(Driver, self).__init__(**kwargs)

    def create(self):
        return ShimadzuMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = ShimadzuMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) == 0:
            return None
        elif len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = samples[0][0]
            msg.add(
                MessageV2.ORDER,
                analysis="DL AQ DP",
                sample_name="{} {}".format(sample.last_name, sample.first_name),
                sample_id=sample.sample_id,
                sample_type="0:Unknown",
                level="0",
                volume="10",
            )

        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        liste_codes = []
        # Traitement du fichier Insight
        control = False
        date = datetime.datetime.now()
        date = date.replace(second=0, microsecond=0)
        if message and message[0].filename == "ID#" and message[1].filename == "Name":
            sample_id = ""
            code_analyze = ""
            for m in message:
                if m.filename == "ID#":
                    sample_id = m.sample_name
                    continue
                elif m.filename == "Name":
                    code_analyze = m.sample_name
                    continue
                elif m.filename == "Flags":
                    liste_codes.append(m.sample_id)
                    liste_codes.extend(m.results)
                    RESULTS_CODES = liste_codes
                    continue
                if sample_id and code_analyze:
                    control = False
                    if self.qc_prefix and sample_id.startswith(self.qc_prefix):
                        sample = worklist.get_or_create(
                            sample_id=sample_id,
                            control_lot=sample_id,
                            type="control"
                        )
                        control = True
                    else:
                        sample = worklist.get_or_create(
                            sample_id=sample_id,
                            type="patient"
                        )
                    for i, result in enumerate(m.results):
                        if not result.strip():
                            continue
                        sample.results.add(
                            code="{}{}{}".format(
                                code_analyze,
                                m.filename,
                                RESULTS_CODES[i].strip()
                            ),
                            value=result.replace(",", ".").strip(),
                            date=date if control else None
                        )
        # Traitement du fichier Export Clam de type csv
        elif message and message[0].filename == "Data Filename" and message[0].sample_name == "Sample Name":
            for m in message:
                if m.filename == "Data Filename":
                    RESULTS_CODES = m.results
                    continue
                if self.qc_prefix and m.sample_id.startswith(self.qc_prefix):
                    sample = worklist.get_or_create(
                        sample_id=m.sample_id,
                        control_lot=m.sample_id,
                        type="control",
                    )
                    control = True
                else:
                    sample = worklist.get_or_create(
                        sample_id=m.sample_id,
                        type="patient",
                    )
                for code in self.code_analyseur.split(";"):
                    if code in m.filename:
                        instrument = code
                    else:
                        instrument = ""
                for i, result in enumerate(m.results):
                    if not result.strip():
                        continue
                    elif RESULTS_CODES[i].strip() == "IS":
                        # Ignorer les résultats de code résultat IS sans intérêt selon les biologistes
                        continue
                    sample.results.add(
                        code="{}".format(RESULTS_CODES[i].strip()),
                        value=result.replace(",", ".").strip(),
                        date=date if control else None,
                        instrument=instrument,
                    )
        return worklist
